/*
 * Copyright (C) 2003-2009 eXo Platform SAS.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.reflext.core;

import org.reflext.api.TypeInfo;
import org.reflext.api.WildcardTypeInfo;

import java.util.List;

/**
 * @author <a href="mailto:julien.viet@exoplatform.com">Julien Viet</a>
 * @version $Revision$
 */
abstract class AbstractWildcardType<T, M, A, P, F> extends AbstractTypeInfo<T, M, A, P, F> implements WildcardTypeInfo {

  public AbstractWildcardType(TypeResolverImpl<T, M, A, P, F> domain) {
    super(domain);
  }

  public String getName() {
    List<TypeInfo> bounds = getUpperBounds();
    if (bounds.isEmpty()) {
      return Object.class.getName();
    } else {
      return bounds.get(0).getName();
    }
  }

  public List<TypeInfo> getUpperBounds() {
    return null;
  }

  public boolean isReified() {
    throw new UnsupportedOperationException("Does it make sense?");
  }

  public final int hashCode() {
    return getUpperBounds().hashCode() ^ getLowerBounds().hashCode();
  }

  public final boolean equals(Object obj) {
    if (obj == this) {
      return true;
    }
    if (obj instanceof WildcardTypeInfo) {
      WildcardTypeInfo that = (WildcardTypeInfo)obj;
      return getUpperBounds().equals(that.getUpperBounds()) && getLowerBounds().equals(that.getLowerBounds());
    }
    return false;
  }
}